# Modified Step Function Model

# Simulation parameters
N <- 50000 # time steps
DELTA <- 0.1 # delta t
MAX <- 50 # max colony size
EPSILON <- 0.1 # extinction level

# Population parameters (We vary b,q, so excluded here)
b0 <- 0.5*DELTA
d0 <- 0.1*DELTA
d <- 0.01*DELTA
alpha <- 0.1*DELTA
m <- 3
eta <- 0.01

# Density of simulations in param space
V <- 9 # number of q points
U <- 20 # number of b points

# Arrays for output form simulation
SIVALUES <- matrix(0,V)
EIVALUES <- matrix(0,V)
QVALUES <- matrix(0,V)
DATA <- matrix(0,U*V,4)

# Calculates density limitation factor
calcphi <- function(POPSIZE) {
  return(1/(1+eta*POPSIZE))
}

# Create matrix
createM <- function(B,D,phi){
  M <- matrix(0,MAX,MAX)
  
  for (i in 1:MAX){
    M[1,i] <- ((1-q)*phi*B[i])
    M[i,i] <- M[i,i]-(q*phi*B[i])-D[i]-(i-1)*alpha
  }
  
  for (i in 1:(MAX-1)){
    M[i,i+1] <- M[i,i+1]+i*alpha
    M[i+1,i] <- M[i+1,i]+q*phi*B[i]
  }
  
  return(M)
}

# Loop over b values
for (u in 1:U) {
  
  b = u*b0
  
  # Modified birth and death vectors
  B <- rep(b,MAX)
  D <- rep(d,MAX)
  for (l in 1:(m-1)) {
    B[l] <- b0 + (l-1)*(b-b0)/(m-1)
    D[l] <- d0 + (l-1)*(d-d0)/(m-1)
  }
  
  indices <- c(1:MAX)
  
  # Loop over q values
  for (v in 1:V){
    
    q <- v/(V+1)
    QVALUES[v] <- q
    
    x0 <- 100   # initialize population vectors
    x <- c(1:MAX)*0; x[1] <- 100
    
    # Run simulation for particular b ranging over q
    for (i in 1:N){
      
      if (min(x) < 0){   # throws error if any entries of x are negative
        print("error negative value")
        break
      }
      
      eusindvcount <- sum(indices*x)   # counts individuals in eusocial colonies
      euscolcount <- sum(x)   # updates colcount
      
      popsize <- (x0+eusindvcount)   # popsize = sum of queens + inviduals in colonies
      phi <- calcphi(popsize)   # updates density dependence
      
      x0 <- x0 + (phi*b0-d0)*x0   # update x0
      
      M <- createM(B,D,phi)   # update M matrix
      x <- x + M%*%x   #update x vector
      
      if (eusindvcount < EPSILON | x0 < EPSILON) {   # stop if either population below epsilon
        break
      }
    }
    
    eusindvcount <- sum(indices*x)
    euscolcount <- sum(x)
    popsize <- (x0+eusindvcount)
    
    print(x0)
    print(eusindvcount)
    
    SIVALUES[v] <- x0
    EIVALUES[v] <- eusindvcount
    
    # Store data points
    DATA[(u-1)*V+v,] = c(q,b/DELTA,SIVALUES[v],EIVALUES[v])
  }
  
  plot(QVALUES, SIVALUES, type="o", col="red", ylim=c(0,max(
    max(SIVALUES),max(EIVALUES))+50), xlab = "Probability to stay, q", 
    ylab = "Individuals", main = paste("Original b = ",b/DELTA))
  lines(QVALUES, EIVALUES, type="o", col="blue", pch=22, lty=2)
  
}

# Plot
DATAPLOT <- matrix(0,U*V,2)
count = 1
for (u in 1:U){
  for (v in 1:V) {
    if (DATA[(u-1)*V+v,4] > DATA[(u-1)*V+v,3]) {
      DATAPLOT[count,1] = DATA[(u-1)*V+v,1]
      DATAPLOT[count,2] = DATA[(u-1)*V+v,2]
      count = count+1
    }
  }
}
DATAPLOT = DATAPLOT[1:(count-1),]
plot(DATAPLOT[,1],DATAPLOT[,2],xlim = c(0.1,0.9), ylim = c(0,10), 
xlab = "q", ylab = "b", main = "Modified Nowak et al. Model")